@@ -2,6 +2,7 @@ from __future__ import absolute_import
 from __future__ import print_function
 from concurrent.futures import Future as ThreadFuture
 from functools import partial
+import copy
 import logging
 import threading
 
@@ -51,7 +52,12 @@ class RmqPublisher(messages.BasePublisherWithReplyQueue):
             body=self._encode(message_dict),
             delivery_mode=topika.DeliveryMode.NOT_PERSISTENT,
         )
-        result = yield self.publish(message, routing_key=defaults.BROADCAST_TOPIC)
+        try:
+            result = yield self.publish(message, routing_key=defaults.BROADCAST_TOPIC)
+        except pika.exceptions.UnroutableError as exception:
+            _LOGGER.debug('Broadcast message sent but was unroutable, probably no queues: %s', str(exception))
+            # Doesn't matter because broadcasts are meant to be fire-and-forget anyway
+            result = True
         raise gen.Return(result)
 
 
@@ -60,11 +66,16 @@ class RmqSubscriber(object):
     Subscriber for receiving a range of messages over RMQ
     """
 
+    BROADCAST_QUEUE_ARGUMENTS = {'x-message-ttl': defaults.MESSAGE_TTL, 'x-expires': defaults.QUEUE_EXPIRES}
+
+    RPC_QUEUE_ARGUMENTS = {'x-message-ttl': defaults.MESSAGE_TTL, 'x-expires': defaults.QUEUE_EXPIRES}
+
     def __init__(self,
                  connection,
                  message_exchange=defaults.MESSAGE_EXCHANGE,
                  decoder=defaults.DECODER,
-                 encoder=defaults.ENCODER):
+                 encoder=defaults.ENCODER,
+                 testing_mode=False):
         """
         Subscribes and listens for process control messages and acts on them
         by calling the corresponding methods of the process manager.
@@ -77,62 +88,80 @@ class RmqSubscriber(object):
         super(RmqSubscriber, self).__init__()
 
         self._connection = connection
-        self._channel = None
-        self._exchange = None
+        self._channel = None  # type: topika.Channel
+        self._exchange = None  # type: topika.Exchange
         self._exchange_name = message_exchange
         self._decode = decoder
+        self._testing_mode = testing_mode
         self._response_encode = encoder
 
         self._rpc_subscribers = {}
         self._broadcast_subscribers = []
+        self._broadcast_queue = None  # type: topika.Queue
+        self._broadcast_consumer_tag = None
 
         self._active = False
 
+    @gen.coroutine
     def add_rpc_subscriber(self, subscriber, identifier):
-        self._rpc_subscribers[identifier] = subscriber
+        # Create an RPC queue
+        rpc_queue = yield self._channel.declare_queue(exclusive=True, arguments=self.RPC_QUEUE_ARGUMENTS)
 
+        yield rpc_queue.bind(self._exchange, routing_key='{}.{}'.format(defaults.RPC_TOPIC, identifier))
+        rpc_queue.consume(partial(self._on_rpc, subscriber))
+        self._rpc_subscribers[identifier] = rpc_queue
+
+    @gen.coroutine
     def remove_rpc_subscriber(self, identifier):
         try:
-            self._rpc_subscribers.pop(identifier)
+            rpc_queue = self._rpc_subscribers.pop(identifier)
         except KeyError:
             raise ValueError("Unknown subscriber '{}'".format(identifier))
+        else:
+            yield rpc_queue.unbind(self._exchange, routing_key='{}.{}'.format(defaults.RPC_TOPIC, identifier))
 
     def add_broadcast_subscriber(self, subscriber):
         self._broadcast_subscribers.append(subscriber)
+        if self._broadcast_consumer_tag is None:
+            # Consume on the broadcast queue
+            self._broadcast_consumer_tag = self._broadcast_queue.consume(self._on_broadcast)
 
     def remove_broadcast_subscriber(self, subscriber):
         self._broadcast_subscribers.remove(subscriber)
+        if not self._broadcast_subscribers:
+            self._broadcast_queue.cancel(self._broadcast_consumer_tag)
+            self._broadcast_consumer_tag = None
 
     def channel(self):
         return self._channel
 
     @gen.coroutine
     def connect(self):
+        """Get a channel and set up all the exchanges/queues we need"""
         if self._channel:
             # Already connected
             return
 
+        exchange_params = copy.copy(EXCHANGE_PROPERTIES)
+
+        if self._testing_mode:
+            exchange_params.setdefault('auto_delete', self._testing_mode)
+
         self._channel = yield self._connection.channel()
-        self._exchange = yield self._channel.declare_exchange(name=self._exchange_name, **EXCHANGE_PROPERTIES)
+        self._exchange = yield self._channel.declare_exchange(name=self._exchange_name, **exchange_params)
 
-        # RPC queue
-        rpc_queue = yield self._channel.declare_queue(
-            exclusive=True, arguments={
-                "x-message-ttl": defaults.MESSAGE_TTL,
-                "x-expires": defaults.QUEUE_EXPIRES
-            })
+        yield self._create_broadcast_queue()
 
-        yield rpc_queue.bind(self._exchange, routing_key='{}.*'.format(defaults.RPC_TOPIC))
-        rpc_queue.consume(self._on_rpc)
+    @gen.coroutine
+    def _create_broadcast_queue(self):
+        """
+        Create and bind the broadcast queue
 
-        # Broadcast queue
-        broadcast_queue = yield self._channel.declare_queue(
-            exclusive=True, arguments={
-                "x-message-ttl": defaults.MESSAGE_TTL,
-                "x-expires": defaults.QUEUE_EXPIRES
-            })
-        yield broadcast_queue.bind(self._exchange, routing_key=defaults.BROADCAST_TOPIC)
-        broadcast_queue.consume(self._on_broadcast)
+        One is used for all broadcasts on this exchange
+        """
+        self._broadcast_queue = yield self._channel.declare_queue(
+            exclusive=True, arguments=self.BROADCAST_QUEUE_ARGUMENTS)
+        yield self._broadcast_queue.bind(self._exchange, routing_key=defaults.BROADCAST_TOPIC)
 
     @gen.coroutine
     def disconnect(self):
@@ -141,35 +170,29 @@ class RmqSubscriber(object):
         self._channel = None
 
     @gen.coroutine
-    def _on_rpc(self, message):
+    def _on_rpc(self, subscriber, message):
         """
-        :param message: The RMQ message
+        :param subscriber: the subscriber function or coroutine that will get the RPC message
+        :param message: the RMQ message
         :type message: :class:`topika.message.IncomingMessage`
         """
         with message.process(ignore_processed=True):
-            identifier = message.routing_key[len('{}.'.format(defaults.RPC_TOPIC)):]
-            receiver = self._rpc_subscribers.get(identifier, None)
-            if receiver is None:
-                message.reject(requeue=True)
-            else:
-                # Tell the sender that we've dealt with it
-                message.ack()
-                msg = self._decode(message.body)
+            # Tell the sender that we've dealt with it
+            message.ack()
+            msg = self._decode(message.body)
 
-                try:
-                    receiver = utils.ensure_coroutine(receiver)
-                    result = yield receiver(self, msg)
-                except Exception as exception:  # pylint: disable=broad-except
-                    # We had an exception in  calling the receiver
-                    yield self._send_response(message.reply_to, message.correlation_id,
-                                              utils.exception_response(exception))
+            try:
+                receiver = utils.ensure_coroutine(subscriber)
+                result = yield receiver(self, msg)
+            except Exception as exception:  # pylint: disable=broad-except
+                # We had an exception in  calling the receiver
+                yield self._send_response(message.reply_to, message.correlation_id, utils.exception_response(exception))
+            else:
+                if concurrent.is_future(result):
+                    yield self._send_future_response(result, message.reply_to, message.correlation_id)
                 else:
-                    if concurrent.is_future(result):
-                        yield self._send_future_response(result, message.reply_to, message.correlation_id)
-                    else:
-                        # All good, send the response out
-                        yield self._send_response(message.reply_to, message.correlation_id,
-                                                  utils.result_response(result))
+                    # All good, send the response out
+                    yield self._send_response(message.reply_to, message.correlation_id, utils.result_response(result))
 
     @gen.coroutine
     def _on_broadcast(self, message):
@@ -182,10 +205,7 @@ class RmqSubscriber(object):
                                    msg[messages.BroadcastMessage.SUBJECT],
                                    msg[messages.BroadcastMessage.CORRELATION_ID])
                 except Exception:  # pylint: disable=broad-except
-                    import traceback
-                    _LOGGER.error("Exception in broadcast receiver!\n"
-                                  "msg: %s\n"
-                                  "traceback:\n%s", msg, traceback.format_exc())
+                    _LOGGER.exception('Exception in broadcast receiver')
 
     @gen.coroutine
     def _send_future_response(self, future, reply_to, correlation_id):
@@ -258,16 +278,11 @@ class RmqCommunicator(object):
 
         self._connection = connection
         self._loop = connection.loop
-        self._connected = False
 
         self._message_subscriber = RmqSubscriber(
-            connection, message_exchange=message_exchange, encoder=encoder, decoder=decoder)
+            connection, message_exchange=message_exchange, encoder=encoder, decoder=decoder, testing_mode=testing_mode)
         self._message_publisher = RmqPublisher(
-            connection,
-            exchange_name=message_exchange,
-            encoder=encoder,
-            decoder=decoder,
-        )
+            connection, exchange_name=message_exchange, encoder=encoder, decoder=decoder, testing_mode=testing_mode)
         self._task_subscriber = tasks.RmqTaskSubscriber(
             connection,
             exchange_name=task_exchange,
@@ -302,16 +317,14 @@ class RmqCommunicator(object):
 
     @gen.coroutine
     def connect(self):
-        if self._connected:
-            return
+        if not self._connection.is_open:
+            yield self._connection.connect()
 
         yield self._message_subscriber.connect()
         yield self._task_subscriber.connect()
         yield self._message_publisher.connect()
         yield self._task_publisher.connect()
 
-        self._connected = True
-
     @gen.coroutine
     def disconnect(self):
         yield self._message_publisher.disconnect()
@@ -320,11 +333,13 @@ class RmqCommunicator(object):
         yield self._task_subscriber.disconnect()
         yield self._connection.close()
 
+    @gen.coroutine
     def add_rpc_subscriber(self, subscriber, identifier):
-        self._message_subscriber.add_rpc_subscriber(subscriber, identifier)
+        yield self._message_subscriber.add_rpc_subscriber(subscriber, identifier)
 
+    @gen.coroutine
     def remove_rpc_subscriber(self, identifier):
-        self._message_subscriber.remove_rpc_subscriber(identifier)
+        yield self._message_subscriber.remove_rpc_subscriber(identifier)
 
     def add_task_subscriber(self, subscriber):
         self._task_subscriber.add_task_subscriber(subscriber)
@@ -359,10 +374,10 @@ class RmqCommunicator(object):
         raise gen.Return(result)
 
     @gen.coroutine
-    def task_send(self, msg):
+    def task_send(self, task, no_reply=False):
         try:
-            response_future = yield self._task_publisher.task_send(msg)
-            raise gen.Return(response_future)
+            result = yield self._task_publisher.task_send(task, no_reply)
+            raise gen.Return(result)
         except pika.exceptions.UnroutableError as exception:
             raise kiwipy.UnroutableError(str(exception))
         except pika.exceptions.NackError as exception:
@@ -370,6 +385,13 @@ class RmqCommunicator(object):
 
 
 class RmqThreadCommunicator(kiwipy.Communicator):
+    """
+    RabbitMQ communicator that runs an event loop on a separate thread to do communication.
+    This also means that heartbeats are not missed and the main program is free to block for
+    as long as it wants.
+    """
+
+    TASK_TIMEOUT = 5.
 
     @classmethod
     def connect(cls,
@@ -444,10 +466,9 @@ class RmqThreadCommunicator(kiwipy.Communicator):
             testing_mode=testing_mode)
         self._loop = self._communicator.loop
         self._communicator_thread = None
-        self._subscribers = {}
 
     def __enter__(self):
-        self.start()
+        self._ensure_running()
         return self
 
     def __exit__(self, exc_type, exc_val, exc_tb):
@@ -457,8 +478,7 @@ class RmqThreadCommunicator(kiwipy.Communicator):
         return self._loop
 
     def start(self):
-        if self._communicator_thread is not None:
-            return
+        assert self._communicator_thread is None, "Already running"
 
         start_future = ThreadFuture()
 
@@ -500,8 +520,8 @@ class RmqThreadCommunicator(kiwipy.Communicator):
             try:
                 with kiwipy.capture_exceptions(stop_future):
                     yield self._communicator.disconnect()
-                    self._loop.stop()
             finally:
+                self._loop.stop()
                 stop_future.set_result(True)
 
         # The stop will end up setting self._communicator_thread to None
@@ -510,49 +530,40 @@ class RmqThreadCommunicator(kiwipy.Communicator):
         stop_future.result()
 
     def add_rpc_subscriber(self, subscriber, identifier):
-        return self._communicator.add_rpc_subscriber(self._convert_callback(subscriber), identifier)
+        coro = partial(self._communicator.add_rpc_subscriber, subscriber, identifier)
+        return self._run_task(coro)
 
     def remove_rpc_subscriber(self, identifier):
-        return self._communicator.remove_rpc_subscriber(identifier)
+        coro = partial(self._communicator.remove_rpc_subscriber, identifier)
+        return self._run_task(coro)
 
     def add_task_subscriber(self, subscriber):
-        converted = self._convert_callback(subscriber)
-        self._communicator.add_task_subscriber(converted)
-        self._subscribers[subscriber] = converted
+        self._communicator.add_task_subscriber(subscriber)
 
     def remove_task_subscriber(self, subscriber):
-        try:
-            converted = self._subscribers.pop(subscriber)
-        except KeyError:
-            raise ValueError("Subscriber '{}' is unknown".format(subscriber))
-        else:
-            self._communicator.remove_task_subscriber(converted)
+        self._communicator.remove_task_subscriber(subscriber)
 
     def add_broadcast_subscriber(self, subscriber):
-        converted = self._convert_callback(subscriber)
-        self._communicator.add_broadcast_subscriber(converted)
-        self._subscribers[subscriber] = converted
+        self._communicator.add_broadcast_subscriber(subscriber)
 
     def remove_broadcast_subscriber(self, subscriber):
-        try:
-            converted = self._subscribers.pop(subscriber)
-        except KeyError:
-            raise ValueError("Subscriber '{}' is unknown".format(subscriber))
-        else:
-            self._communicator.remove_broadcast_subscriber(converted)
+        self._communicator.remove_broadcast_subscriber(subscriber)
+
+    def task_send(self, task, no_reply=False):
+        self._ensure_running()
+        future = self._send_message(partial(self._communicator.task_send, task, no_reply))
+        if no_reply:
+            return None
 
-    def task_send(self, msg):
-        self.start()
-        future = self._send_message(partial(self._communicator.task_send, msg))
         return self.tornado_to_kiwi_future(future)
 
     def rpc_send(self, recipient_id, msg):
-        self.start()
+        self._ensure_running()
         future = self._send_message(partial(self._communicator.rpc_send, recipient_id, msg))
         return self.tornado_to_kiwi_future(future)
 
     def broadcast_send(self, body, sender=None, subject=None, correlation_id=None):
-        self.start()
+        self._ensure_running()
         return self._send_message(
             partial(
                 self._communicator.broadcast_send,
@@ -561,72 +572,17 @@ class RmqThreadCommunicator(kiwipy.Communicator):
                 subject=subject,
                 correlation_id=correlation_id))
 
-    def wait_for(self, future, timeout=None):
-        self.start()
-        thread_result = ThreadFuture()
-        do_copy = partial(kiwipy.copy_future, target=thread_result)
-        future.add_done_callback(do_copy)
-
-        try:
-            return thread_result.result(timeout=timeout)
-        except concurrent.futures.TimeoutError:
-            future.remove_done_callback(do_copy)
-            raise kiwipy.TimeoutError()
-
     def _send_message(self, coro):
-        send_future = ThreadFuture()
+        send_future = kiwipy.Future()
 
         @gen.coroutine
         def do_task():
             with kiwipy.capture_exceptions(send_future):
-                result_future = yield coro()
-                send_future.set_result(result_future)
+                send_future.set_result((yield coro()))
 
         self._loop.add_callback(do_task)
         return send_future.result()
 
-    def _convert_callback(self, callback):
-        """
-        :param callback: The callback to convert
-        :return: A new function that conforms to that which a Communicator expects
-        """
-
-        def converted(_comm, *args, **kwargs):
-            result = callback(self, *args, **kwargs)
-            if isinstance(result, kiwipy.Future):
-                result = self.kiwi_to_tornado_future(result)
-            return result
-
-        return converted
-
-    def kiwi_to_tornado_future(self, kiwi_future):
-        """
-        Convert a kiwipy future to a tornado future
-
-        :param kiwi_future: the kiwipy future to convert
-        :type kiwi_future: :class:`kiwipy.Future`
-        :return: the tornado future
-        :rtype: :class:`tornado.concurrent.Future`
-        """
-        tornado_future = concurrent.Future()
-
-        def done(done_future):
-            if done_future.cancelled():
-                tornado_future.cancel()
-
-            with kiwipy.capture_exceptions(tornado_future):
-                result = done_future.result()
-                if isinstance(result, kiwipy.Future):
-                    result = self.kiwi_to_tornado_future(result)
-
-                # Schedule the callback to set the result because this method may be called
-                # from outside the event loop in which case any done callbacks would be called
-                # using the wrong event loop
-                self.loop().add_callback(tornado_future.set_result, result)
-
-        kiwi_future.add_done_callback(done)
-        return tornado_future
-
     def tornado_to_kiwi_future(self, tornado_future):
         """
         Convert a tornado future to a kiwipy future
@@ -654,6 +610,31 @@ class RmqThreadCommunicator(kiwipy.Communicator):
         self.loop().add_future(tornado_future, done)
         return kiwi_future
 
+    def _create_task(self, coro):
+        """
+        Create a task to be run on our event loop
+
+        :param coro: the coroutine to run
+        :return: a future corresponding to the execution of the coroutine
+        :rtype: :class:`tornado.concurrent.Future`
+        """
+        return utils.create_task(coro, self._loop)
+
+    def _run_task(self, coro):
+        """
+        Run a coroutine on the event loop and return the result.  It may take a little time for the loop
+        to get around to scheduling it so we use a timeout as set by the TASK_TIMEOUT class constant.
+
+        :param coro: the coroutine to run
+        :return: the result of running the coroutine
+        """
+        return self.tornado_to_kiwi_future(self._create_task(coro)).result(timeout=self.TASK_TIMEOUT)
+
+    def _ensure_running(self):
+        if self._communicator_thread is not None:
+            return
+        self.start()
+
 
 def connect(connection_params=None,
             connection_factory=topika.connect_robust,
