@@ -1,4 +1,5 @@
 from __future__ import absolute_import
+import collections
 import logging
 import uuid
 import sys
@@ -16,11 +17,14 @@ _LOGGER = logging.getLogger(__name__)
 
 __all__ = ['RmqTaskSubscriber', 'RmqTaskPublisher']
 
+TaskBody = collections.namedtuple('TaskBody', ('task', 'no_reply'))
+
 
 class RmqTaskSubscriber(messages.BaseConnectionWithExchange):
     """
     Listens for tasks coming in on the RMQ task queue
     """
+    TASK_QUEUE_ARGUMENTS = {"x-message-ttl": defaults.TASK_MESSAGE_TTL}
 
     def __init__(self,
                  connection,
@@ -44,7 +48,7 @@ class RmqTaskSubscriber(messages.BaseConnectionWithExchange):
         :param encoder: A response encoder
         """
         super(RmqTaskSubscriber, self).__init__(
-            connection, exchange_name=exchange_name, exchange_params=exchange_params)
+            connection, exchange_name=exchange_name, exchange_params=exchange_params, testing_mode=testing_mode)
 
         self._task_queue_name = queue_name
         self._testing_mode = testing_mode
@@ -60,9 +64,14 @@ class RmqTaskSubscriber(messages.BaseConnectionWithExchange):
 
     def add_task_subscriber(self, subscriber):
         self._subscribers.append(subscriber)
+        if self._consumer_tag is None:
+            self._consumer_tag = self._task_queue.consume(self._on_task)
 
     def remove_task_subscriber(self, subscriber):
         self._subscribers.remove(subscriber)
+        if not self._subscribers:
+            self._task_queue.cancel(self._consumer_tag)
+            self._consumer_tag = None
 
     @gen.coroutine
     def connect(self):
@@ -73,56 +82,72 @@ class RmqTaskSubscriber(messages.BaseConnectionWithExchange):
         yield super(RmqTaskSubscriber, self).connect()
         yield self.channel().set_qos(prefetch_count=self._prefetch_count, prefetch_size=self._prefetch_size)
 
-        # Set up task queue
+        yield self._create_task_queue()
+
+    @gen.coroutine
+    def _create_task_queue(self):
+        """Create and bind the task queue"""
+        arguments = dict(self.TASK_QUEUE_ARGUMENTS)
+        if self._testing_mode:
+            arguments['x-expires'] = defaults.TEST_QUEUE_EXPIRES
+
+        # x-expires means how long does the queue stay alive after no clients
+        # x-message-ttl means what is the default ttl for a message arriving in the queue
         self._task_queue = yield self._channel.declare_queue(
             name=self._task_queue_name,
             durable=not self._testing_mode,
             auto_delete=self._testing_mode,
-            arguments={"x-message-ttl": defaults.TASK_MESSAGE_TTL})
-        # x-expires means how long does the queue stay alive after no clients
-        # x-message-ttl means what is the default ttl for a message arriving in the queue
+            arguments=arguments)
         yield self._task_queue.bind(self._exchange, routing_key=self._task_queue.name)
 
-        self._consumer_tag = self._task_queue.consume(self._on_task)
-
     @gen.coroutine
     def _on_task(self, message):
         """
         :param message: The topika RMQ message
         :type message: :class:`topika.IncomingMessage`
         """
+        # pylint: disable=too-many-branches
+
         with message.process(ignore_processed=True):
             handled = False
-            task = self._decode(message.body)
+            # Decode the message tuple into a task body for easier use
+            task_body = TaskBody(*self._decode(message.body))  # type: TaskBody
             for subscriber in self._subscribers:
                 try:
                     subscriber = utils.ensure_coroutine(subscriber)
-                    result = yield subscriber(self, task)
-                    # If a task returns a future it is not considered until the chain of futrues
+                    result = yield subscriber(self, task_body.task)
+
+                    # If a task returns a future it is not considered done until the chain of futrues
                     # (i.e. if the first future resolves to a future and so on) finishes and produces
                     # a concrete result
                     while concurrent.is_future(result):
-                        yield self._send_response(utils.pending_response(), message)
+                        if not task_body.no_reply:
+                            yield self._send_response(utils.pending_response(), message)
                         result = yield result
                 except kiwipy.TaskRejected:
+                    # Keep trying to find one that will accept the task
                     continue
                 except kiwipy.CancelledError as exception:
-                    reply_body = utils.cancelled_response(str(exception))
+                    if not task_body.no_reply:
+                        reply_body = utils.cancelled_response(str(exception))
                     handled = True  # Finished
                 except KeyboardInterrupt:  # pylint: disable=try-except-raise
                     raise
                 except Exception as exc:  # pylint: disable=broad-except
                     _LOGGER.debug('There was an exception in task %s:\n%s', exc, traceback.format_exc())
-                    reply_body = utils.exception_response(sys.exc_info()[1:])
+                    if not task_body.no_reply:
+                        reply_body = utils.exception_response(sys.exc_info()[1:])
                     handled = True  # Finished
                 else:
                     # Create a reply message
-                    reply_body = utils.result_response(result)
+                    if not task_body.no_reply:
+                        reply_body = utils.result_response(result)
                     handled = True  # Finished
 
                 if handled:
                     message.ack()
-                    yield self._send_response(reply_body, message)
+                    if not task_body.no_reply:
+                        yield self._send_response(reply_body, message)
                     break  # Done, do break the loop
 
             if not handled:
@@ -178,16 +203,26 @@ class RmqTaskPublisher(messages.BasePublisherWithReplyQueue):
         self._task_queue_name = task_queue_name
 
     @gen.coroutine
-    def task_send(self, msg):
+    def task_send(self, task, no_reply=False):
         """
         Send a task for processing by a task subscriber
-        :param msg: The task payload
+        :param task: The task payload
+        :param no_reply: Don't send a reply containing the result of the task
+        :type no_reply: bool
         :return: A future representing the result of the task
         :rtype: :class:`tornado.concurrent.Future`
         """
-        task_msg = topika.Message(
-            body=self._encode(msg), correlation_id=str(uuid.uuid4()), reply_to=self._reply_queue.name)
-        published, result_future = yield self.publish_expect_response(
-            task_msg, routing_key=self._task_queue_name, mandatory=True)
+        # Build the full message body and encode as a tuple
+        body = self._encode((task, no_reply))
+        # Now build up the full topika message
+        task_msg = topika.Message(body=body, correlation_id=str(uuid.uuid4()), reply_to=self._reply_queue.name)
+
+        result_future = None
+        if no_reply:
+            published = yield self.publish(task_msg, routing_key=self._task_queue_name, mandatory=True)
+        else:
+            published, result_future = yield self.publish_expect_response(
+                task_msg, routing_key=self._task_queue_name, mandatory=True)
+
         assert published, "The task was not published to the exchange"
         raise gen.Return(result_future)
